///////
//  A Win 95 / NT Application for working with the Clipboard.
///////
//
//   This program allows users to work with the clipboard from
// the command line.
//
//   There are three options available :-
//  
//   1. Print the contents of the clipboard to the console
//   2. Clear the clipboard
//   3. Set the clipboard contents to be the contents of a file
//
//   This could be used from within batch files, if programs are
//  capable of reading from input files, but not the clipboard,
//  simply do something like :-
//   
//   clipboard > t.txt
//   process t.txt 
//
//   to work with the clipboard contents.
//
///////
//
// Copyright (c) Steve Kemp 1998, skx@tardis.ed.ac.uk
//
// http://www.tardis.ed.ac.uk/~skx/
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////

// Includes
#include "Windows.h"
#include "stdio.h"

// Defines

// Function Prototypes
 // Display some help to the user
void printHelp();
 // Print the contents of the clipboard to STDOUT
void printClipboard();
 // Copy the contents of a file to the clipboard
void copyFileToClipboard( char *fileName );



/**
  * Entry point to the code
  *  Parse the arguments, and do the appropriate thing, this could be :-
  *  1. Print the contents of the clipboard
  *  2. Clear the clipboard
  *  3. Set the clipboard to contain the contents of the named file.
  * 
  */
int main( int argc, char *argv[] )
{
  // Used in indexing the arguments
  int i = 0;

  // No arguments so we can clear the clipboard
  if ( argc == 1 )
  { printClipboard();
    return 0;
  }

  // Are we going to clear the clipboard
  int clearClipBoard = 0;

  // See if the user wants us to clear the clipboard
  for ( i = 1; i < argc; i++ )
  { if ( stricmp( argv[ i ], "/c" ) == 0 )
      clearClipBoard = 1;
    if ( stricmp( argv[ i ], "/clear" ) == 0 )
      clearClipBoard = 1;
    if ( stricmp( argv[ i ], "/?" ) == 0 )
    { printHelp();
      return 0;
    }
    if ( stricmp( argv[ i ], "/h" ) == 0 )
    { printHelp();
      return 0;
    }
  }

  // Okay, we got a "Clear Clipboard" flag, so clear it, and exit
  if ( clearClipBoard )
  { 
    // Open the clipboard
    OpenClipboard( NULL );
    
    // Empty the clipboard
    if ( !EmptyClipboard() )
      printf( "Couldn't clear clipboard.\n" );

    // Close it.
    CloseClipboard();
    return 0;
  }

  // Copy named files to the clipboard
  for ( i = 1; i < argc; i++ )
  {
    if ( stricmp( argv[ i ], "/clear" ) != 0 )
    {
      if ( stricmp( argv[ i ], "/c" ) != 0 )
      { copyFileToClipboard( argv[ i ] );
        return 0;
      } 
    }
  }
  
  return 0;
}


/**
  * Display basic usage instructions to the user.
  */
void printHelp()
{
  printf( "Clipboard v1.0 by Steve Kemp <skx@tardis.ed.ac.uk>\n" );
  printf( "\n" );
  printf( "Usage: clipboard          - Displays the contents of the clipboard\n" );
  printf( "       clipboard /clear   - Clears the contents of the clipboard\n" );
  printf( "       clipboard fileName - Copy the contents of the file to the clipboard\n" );
  printf( "       clipboard /?       - Shows this message\n" );
}


/**
  * Print the contents of the clipboard to stdout.
  */
void printClipboard()
{
  // Handle to locked memory containg clipboard contents
  HANDLE hMem;

  // Open the clipboard
  OpenClipboard( NULL );

  // Retrieve the text
  if ( hMem = GetClipboardData( CF_TEXT ) )
  { LPSTR lpMem = (char *)GlobalLock( hMem );
    printf( "%s\n", lpMem );
    GlobalUnlock( hMem );
  }

  // Close the clipboard
  CloseClipboard();

}


/**
  * Copy the contents of the named file to the clipboard.
  */
void copyFileToClipboard( char *fileName )
{

  DWORD dwSize, dwRead;
  LPVOID lpBuffer;
  HANDLE hMem;


  // Open the file
  HANDLE hfile = CreateFile( fileName , GENERIC_READ, FILE_SHARE_READ, 
                             NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 
                             NULL );
	
  if ( hfile == INVALID_HANDLE_VALUE )
  { printf( "Couldn't open input file '%s'\n", fileName );
    return;
  } 

  dwSize = GetFileSize( hfile, NULL );
  dwSize++;
  dwSize++;

  // Allocate enough memory to read the file in one go.	
  hMem = GlobalAlloc(GHND, dwSize);

  if ( hMem == NULL )
  { printf( "Couldn't allocate enough memory to read the input file\n(Needed %d bytes)\n", dwSize );
    return;
  } 

  // Lock the memory so that it won't be free'd when we exit
  lpBuffer = GlobalLock( hMem );
  
  if ( lpBuffer == NULL )
  { printf( "Couldn't lock the memory to contain the clipboard data\n" );
    return;
  }

  // Read the input file.
  ReadFile( hfile, lpBuffer, dwSize - 1, &dwRead, NULL );	

  // Open the clipboard
  OpenClipboard( NULL );

  // Print the contents of the file if you're curious
  // printf( "Data is %s\n", lpBuffer );

  // Set the contents of the clipboard
  SetClipboardData( CF_TEXT, lpBuffer );

  // Close the clipboard
  CloseClipboard();

  // Close the input file
  CloseHandle( hfile );
}

